import os


def create_package(path):
    if not os.path.exists(path):
        os.makedirs(path)
    create_module(path)


def create_module(path, name=None, content=None):
    name = name if name is not None else '__init__'
    file = os.path.join(path, f'{name}.py')
    content = content if content is not None else ''
    if not os.path.exists(file):
        with open(file, 'w') as f:
            f.write(content)



from importlib import import_module

from falcon import API

from falcon_core.config import settings

from falcon_core.utils import import_object, load_middleware, flatten


def get_wsgi_application():
    media_type = import_object(settings.DEFAULT_MEDIA_TYPE)
    request_type = import_object(settings.REQUEST_TYPE)
    response_type = import_object(settings.RESPONSE_TYPE)
    middleware = [load_middleware(m) for m in settings.MIDDLEWARE]
    router = import_object(settings.ROUTER)
    independent_middleware = settings.INDEPENDENT_MIDDLEWARE

    application = API(media_type=media_type, request_type=request_type, response_type=response_type,
                      middleware=middleware, router=router(), independent_middleware=independent_middleware)

    for k, v in settings.ROUTER_CONVERTERS.items():
        application.router_options.converters[k] = import_object(v)

    for route in flatten(import_module(settings.ROUTES).routes):
        application.add_route(route.uri_template, route.resource, **route.kwargs)

    return application



import json

from falcon import HTTPNotFound


class Resource:
    storage = {}

    def on_get(self, req, resp, **kwargs):
        raise HTTPNotFound

    def on_post(self, req, resp, **kwargs):
        raise HTTPNotFound

    def on_put(self, req, resp, **kwargs):
        raise HTTPNotFound

    def on_delete(self, req, resp, **kwargs):
        raise HTTPNotFound


def dict_from_req(req):
    if req.content_length:
        try:
            return json.load(req.stream)
        except json.JSONDecodeError:
            return {}
    return {}


class JSONResourceBase(type):
    def __new__(mcs, name, bases, attrs):
        http_handlers = ('on_get', 'on_post', 'on_put', 'on_delete')

        for obj_name, obj in attrs.items():
            if obj_name.startswith(http_handlers):
                def http_handler(instance, req, resp, **kwargs):
                    req.context['data'] = dict_from_req(req)
                    obj(instance, req, resp, **kwargs)

                attrs[obj_name] = http_handler

        return super().__new__(mcs, name, bases, attrs)


class JSONResource(Resource, metaclass=JSONResourceBase):
    pass



import os
import sys
import pkgutil
from importlib import import_module

from falcon_core.config import ENVIRONMENT_VARIABLE, settings, global_settings


def find_commands(management_dir):
    command_dir = os.path.join(management_dir, 'commands')
    return [name for _, name, is_pkg in pkgutil.iter_modules([command_dir])
            if not is_pkg]


def load_command_class(app_name, name):
    module = import_module(f'{app_name}.management.commands.{name}')
    return module.Command()


def execute_command_from_argv(argv):
    if len(argv) > 1:
        command = argv[1]
        commands = {}
        if os.environ.get(ENVIRONMENT_VARIABLE):
            apps_names = settings.INSTALLED_APPS
        else:
            apps_names = global_settings.INSTALLED_APPS
        for app_name in apps_names:
            app = import_module(app_name)
            management_dir = os.path.join(os.path.dirname(app.__file__), 'management')
            if os.path.exists(management_dir):
                commands.update({name: load_command_class(app_name, name) for name in find_commands(management_dir)})
        commands[command].run_from_argv(argv)


def execute_from_command_line(argv=None):
    execute_command_from_argv(argv or sys.argv)



from setuptools import setup, find_packages


def read(file):
    with open(file) as f:
        return f.read()


setup(
    name='falcon_core',
    version='0.0.24',
    description='Falcon Core Inspired by Django for Falcon API Framework.',
    long_description=read('README.md'),
    long_description_content_type="text/markdown",
    author='Maksym Sichkaruk',
    author_email='maxim.18.08.1997@gmail.com',
    license='MIT',
    url='https://github.com/Maksych/falcon_core',
    packages=find_packages(
        include=['falcon_core.*', 'falcon_core'],
        exclude=['tests.*', 'tests'],
    ),
    include_package_data=True,
    scripts=['falcon_core/bin/falcon-core.py'],
    entry_points={'console_scripts': [
        'falcon-core = falcon_core.management:execute_from_command_line',
    ]},
    zip_safe=False,
    install_requires=['falcon', 'waitress', 'gunicorn'],
    classifiers=[
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
    ],
)
